
#include "xparameters.h"
#include "stdio.h"
#include "sleep.h"
#include "radio_controller_basic.h"
#include "radio_controller_ext.h"
#include "radio_controller_adv.h"
#include "radio_controller_5ghz.h"
#include "EEPROM.h"
#include "warp_hw_ver.h"
#include "warp_v4_userio.h"

#define USERIO_BASEADDR XPAR_USERIO_BASEADDR

void userio_init();
int radio_init();
void userio_example();

//Assume a 2-radio kit, with WARP Radio Boards v1.4 in slots 2 and 3
unsigned int radios = RADIO2_ADDR | RADIO3_ADDR;

int main() {

	print("\f");
	print("WARP v2 Template Project - Example Application\n");

	userio_init();
	radio_init();
	userio_example();

	return 0;
}


//Simple demonstration of the user IO on FPGA Board v2
// This function runs forever, reading the user inputs (DIP switch and push buttons)
// and updating the user outputs (hex displays and LEDs) based on the inputs
//Remember the "down" button (SW6) is tied to the PPC reset
void userio_example() {
	int i;

		xil_printf("\tCurrent Count: 0x  ");
		for(i=0; i<4096; i++) {
			WarpV4_UserIO_WriteNumber_LeftHex(USERIO_BASEADDR, (i>>8&0xF), 0);
			WarpV4_UserIO_WriteNumber_MiddleHex(USERIO_BASEADDR, (i>>4)&0xF, 0);
			WarpV4_UserIO_WriteNumber_RightHex(USERIO_BASEADDR, i&0xF, 0);
			WarpV4_UserIO_Leds(USERIO_BASEADDR, i);
			xil_printf("\b\b%02x", i);

			usleep(250000);
		}
		xil_printf("\n");
		return;
}

void userio_init() {

	//Set all hex displays to auto-map mode
	WarpV4_UserIO_NumberMode_LeftHex(USERIO_BASEADDR);
	WarpV4_UserIO_NumberMode_MiddleHex(USERIO_BASEADDR);
	WarpV4_UserIO_NumberMode_RightHex(USERIO_BASEADDR);

	//Set all hex displays to show "0"
	WarpV4_UserIO_WriteNumber_LeftHex(USERIO_BASEADDR, 0, 0);
	WarpV4_UserIO_WriteNumber_MiddleHex(USERIO_BASEADDR, 0, 0);
	WarpV4_UserIO_WriteNumber_RightHex(USERIO_BASEADDR, 0, 0);

	//Turn off all user LEDs
	WarpV4_UserIO_Leds(USERIO_BASEADDR, 0);
}

int radio_init() {
	//Initialize the radio transceivers
	WarpRadio_v1_Reset((unsigned int*)XPAR_RADIO_CONTROLLER_0_BASEADDR, 2);

	//Set some sane defaults for the MAX2829 transceivers
	WarpRadio_v1_SetCenterFreq2GHz(11, radios);
	WarpRadio_v1_SoftwareTxGainControl(1, radios);
	WarpRadio_v1_SoftwareRxGainControl(1, radios);
	WarpRadio_v1_RxHpSoftControlEnable(radios);
	WarpRadio_v1_RxLpfCornFreqCoarseAdj(1, radios);
	WarpRadio_v1_TxLpfCornFreqCoarseAdj(1, radios);

	//Ensure both transceivers are in standby (no Tx or Rx)
	WarpRadio_v1_TxRxDisable(radios);

	return 0;
}
